/* -
 * Copyright (c) 2002, Ramsey G. Brenner. <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * The contents of this file are subject to the terms of the XORSuite
 * Public Source License (the "License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License
 * at http://rgbrenner.cjb.net/projects/xpsl.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 */

extern char *cwd;
extern char *tmp_dir;
extern char *prog_name;

HASH_DATA    hdata;

void
set_hash(HASH_ALGO halgo)
{
    hdata.end = end_hash;
    hdata.zero = zero_hash;

    switch(halgo) {
      case MD:
        hdata.algo = MD;
        hdata.digest_length = MD5_DIGEST_LENGTH;
        hdata.c_size = sizeof(MD5_CTX);
        hdata.c = &hdata.md_c;
        hdata.hash = hdata.md_hash;
        hdata.md = hdata.md_md;
        hdata.init = md_init_hash;
        hdata.update = md_update_hash;
        hdata.final = md_final_hash;
        break;

      case SH:
      default:
        hdata.algo = SH;
        hdata.digest_length = SHA_DIGEST_LENGTH;
        hdata.c_size = sizeof(SHA_CTX);
        hdata.c = &hdata.sh_c;
        hdata.hash = hdata.sh_hash;
        hdata.md = hdata.sh_md;
        hdata.init = sh_init_hash;
        hdata.update = sh_update_hash;
        hdata.final = sh_final_hash;
        break;
    }
}

/* Return the hash type in a string */
char *
get_hash_algo(void)
{
    switch(hdata.algo) {
      case MD:
        return("MD5");

      case SH:
      default:
        return("SHA-1");
    }
}

/* Zero out hash, md, and c */
void
zero_hash(char **hash, unsigned char **md, void *c)
{
    if(hash != NULL) {
      memset(*hash, 0, (hdata.digest_length * 2) + 1);
    }

    if(md != NULL) {
      memset(*md, 0, hdata.digest_length);
    }

    if(c != NULL) {
      memset(c, 0, hdata.c_size);
    }
}

/* Init c */
void
sh_init_hash(void)
{
    SHA1_Init(hdata.c);
}

/* Init c */
void
md_init_hash(void)
{
    MD5_Init(hdata.c);
}

/* Update c with one byte */
void
sh_update_hash(unsigned char *byte)
{
    SHA1_Update(hdata.c, byte, 1);
}

/* Update c with one byte */
void
md_update_hash(unsigned char *byte)
{
    MD5_Update(hdata.c, byte, 1);
}

/* Final */
void
sh_final_hash(void)
{
    SHA1_Final(hdata.md, hdata.c);
}

/* Final */
void
md_final_hash(void)
{
    MD5_Final(hdata.md, hdata.c);
}

/* Convert the hex string md to a printable string */
void
end_hash(void)
{
    int    i;

    for(i = 0; i < hdata.digest_length; i++) {
      sprintf(hdata.hash + i + i, "%02x", hdata.md[i]);
    }
}

/* Hash file `fn' and return its hash */
char *
hash_file(const char *fn)
{
    FILE            *fp;
    unsigned char    byte;

    /* Open file */
    fp = Fopen(fn, "r");

    /* Start hashing */
    hdata.zero(&hdata.hash, &hdata.md, hdata.c);
    hdata.init();
    for(;;) {
      if(Rbyte(&byte, fp)) {
        Fclose(fp);
        break;
      }
      hdata.update(&byte);
    }
    hdata.final();
    hdata.zero(NULL, NULL, hdata.c);
    hdata.end();
    Fclose(fp);
    return(hdata.hash);
}

/* Check if the hash in the filename
   equals its real hash */
void
chk_hash(const char *fn)
{
    int          i;
    char        *hash,
                *fhash,
                *path,
                *filename;
    HASH_ALGO    halgo_bckup;
    enum {
      MISMATCH,
      MATCH
    };

    /* Init htype_bckup */
    halgo_bckup = hdata.algo;

    /* Set filename */
    path = get_path(fn);
    filename = path + strlen(path) + 1;

    /* Set hash type and extract hash from filename */
    if(strncmp(filename, "sha1-", strlen("sha1-")) == 0) {
      set_hash(SH);
      fhash = filename + strlen("sha1-");
    } else if(strncmp(filename, "md5-", strlen("md5-")) == 0) {
      set_hash(MD);
      fhash = filename + strlen("md5-");
    } else {
      /* Assume user renamed file */
      /* Restore hash_algo */
      set_hash(halgo_bckup);
      Free(path);
      return;
    }

    /* Remove postfix */
    for(i = strlen(fhash) - 1; i >= 0; i--) {
      if(fhash[i] == '.') {
        fhash[i] = NULL;
        break;
      }
    }

    /* Hash file */
    hash = hash_file(fn);

    /* Restore hash_algo */
    set_hash(halgo_bckup);

    /* Compare hashes */
    if(strcmp(hash, fhash) == 0) {
    } else {
      Err(HASH_MISMATCH, fn);
    }

    Free(path);
    return;
}

/* Rename file with the format
   <hash type>-<hash>.<postfix> */
char *
hash_rename(const char *fn, const char *postfix)
{
    char  *hash_str,
          *new_name;

    /* Set hash_str */
    switch(hdata.algo) {
      case MD:    /* MD5 */
        hash_str = "md5";
        break;

      case SH:
      default:    /* SHA-1 */
        hash_str = "sha1";
        break;
    }

    /* Create new file name */
    Asprintf(&new_name, "%s%s-%s.%s",
        cwd, hash_str, hash_file(fn), postfix);

    /* Rename file */
    if(rename(fn, new_name) < 0) {
      Free(new_name);
      Err(RENAME, fn);
    }

    return(new_name);
}
