/* -
 * Copyright (c) 2002, Ramsey G. Brenner. <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * The contents of this file are subject to the terms of the XORSuite
 * Public Source License (the "License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License
 * at http://rgbrenner.cjb.net/projects/xpsl.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 */

extern char *cwd;
extern char *tmp_dir;
extern char *prog_name;

/* Return length of d */
/* Ex: d == 12345, len == 5 */
/* d must be positive */
int
dlen(int d)
{
    char   len = 1;

    if(d < 0) {
      Errx("%d must be positive", d);
    }

    for(; d > 10; len++) {
      d /= 10;
    }

    return(len);
}

/* Our atoi() */
int
Atoi(const char *nptr)
{
    long  d;

    d = strtol(nptr, (char **)NULL, 10);
    if(d == LONG_MIN || d == LONG_MAX) {
      Errx("%s: %s",
        nptr, strerror(errno));
    }
    return((int)d);
}

/* Check if entire string is a digit */
tf
strisdigit(const char *s)
{
    int i;

    for(i = 0; s[i] != NULL; i++) {
      if(!isdigit(s[i])) {
        return(FALSE);
      }
    }
    return(TRUE);
}

/* Ask user yes/no question */
int
ask_user(const char *msg)
{
    int  c,
         tmp,
         answer;

    answer = 0;

    /* Print question */
    fprintf(stderr, "%s", msg);

    for(;;) {
      /* Grab char */
      if((c = getchar()) == EOF) {
        perror(NULL);
        exit(EXIT_FAILURE);
      }

      /* Parse char */
      switch(c) {
        case 'Y':
        case 'N':
          c = tolower(c);
        case 'y':
        case 'n':
        case '\n': /* User selected default answer */
          answer = c;

          /* Read the rest of the line */
          /* Get char until '\n' */
          for(tmp = answer; tmp != '\n';) {
            if((tmp = getchar()) == EOF) {
              perror(NULL);
              exit(EXIT_FAILURE);
            }
          }

          return(answer);
      }

      /* User didn't answer correctly */
      continue;
    }

    return(answer);
}

/* Returns the type of the option */
opt_t
get_opt_type(const char *arg)
{
    /* if arg is in format --<arg>, remove extra '-' */
    if(arg[0] == '-' && arg[1] == '-') {
      arg++;
    }

    /* These options do not take an argument,
       and use strcmp() */
    if(strcmp(arg, "-sha1") == 0) {
      return(HASH_SHA1);
    } else if(strcmp(arg, "-md5") == 0) {
      return(HASH_MD5);

    } else if(strcmp(arg, "-ssl") == 0) {
      return(BGEN_SSL);
    } else if(strcmp(arg, "-urandom") == 0) {
      return(BGEN_URANDOM);

    } else if(strcmp(arg, "-f") == 0) {
      return(FFLAG);

    /* These options do take an argument,
       and use strncmp() */
    } else if(strncmp(arg, "-n", 2) == 0) {
      return(N_OPT);

    } else if(strncmp(arg, "-o", 2) == 0) {
      return(OUTPUT_FILE);

    } else if(strncmp(arg, "-s", 2) == 0) {
      return(SIZE);

    /* Else */
    } else {
      return(OTHER);
    }
}

/* Our getopt() */
char *
get_opt(char **arg, int *i)
{
    int    arglen;

    if((*arg)[0] == '-' && (*arg)[1] == '-') {
      arglen = 3;
    } else {
      arglen = 2;
    }

    if(strlen(arg[0]) > arglen) {
      return(arg[0] + arglen);
    } else {
      (*i)++;
      return(arg[1]);
    }
}

/* atexit() */
void
AtExit(void (*function)(void))
{
    if(atexit(function) < 0) {
      perror(NULL);
      exit(EXIT_FAILURE);
    }
    return;
}

/* Replacement for the non-standard asprintf() function */
/* If you dont have it on your system, think sprintf()
   but with dynamic memory allocation */
/* This version only supports int, char, and char * */
void
Asprintf(char **ret, const char *fmt, ...)
{
    int      d;
    char     c,
            *s,
            *ptr,
            *tmp;
    size_t   len,
             nbytes;
    va_list  ap;

    /* Init */
    nbytes = 1;
    ptr = Malloc(1);
    *ptr = NULL;
    va_start(ap, fmt);

    /* Parse fmt */
    for(; *fmt != NULL; fmt++) {
      if(*fmt == '%') {
        fmt++;
        switch(*fmt) {
          case 'd':
            d = va_arg(ap, int);
            len = dlen(d);
            ptr = Realloc(ptr, nbytes + len);
            sprintf(ptr + strlen(ptr), "%d", d);
            nbytes += len;
            break;
          case 'c':
            c = va_arg(ap, int);
            ptr = Realloc(ptr, nbytes + 1);
            sprintf(ptr + strlen(ptr), "%c", c);
            nbytes++;
            break;
          case 's':
            s = va_arg(ap, char *);
            len = strlen(s);
            ptr = Realloc(ptr, nbytes + len);
            sprintf(ptr + strlen(ptr), "%s", s);
            nbytes += len;
            break;
        }
      } else {
        if((tmp = strchr(fmt, '%')) == NULL) {
          len = strlen(fmt);
        } else {
          len = tmp - fmt;
        }
        ptr = Realloc(ptr, nbytes + len);
        strncpy(ptr + strlen(ptr), fmt, len);
        ptr[nbytes + len - 1] = NULL;
        nbytes += len;
        fmt += len - 1;
      }
    }

    /* Assign ptr to *ret */
    *ret = ptr;

    /* Cleanup and return */
    va_end(ap);
    return;
}
